/*
 *   file: form_work.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "form.h"
#include "form_work.h"
#include "app.h"
#include "groups_cb.h"
#include "models.h"
#include "types.h"
#include "utils.h"


struct _RubricaFormWorkPrivate {
  RubricaGroupSet* set;

  GtkWidget* role;
  GtkWidget* org;
  GtkWidget* dep;
  GtkWidget* subdep;
  GtkWidget* manager;
  GtkWidget* mphone;
  GtkWidget* helper;
  GtkWidget* cphone;
  
  gboolean dispose_has_run;
};


static void rubrica_form_work_class_init (RubricaFormWorkClass* klass);
static void rubrica_form_work_init       (RubricaFormWork* obj);

static void rubrica_form_work_dispose    (RubricaFormWork* obj);
static void rubrica_form_work_finalize   (RubricaFormWork* obj);


static void form_work_extract_data (RubricaFormWork* form, RubricaCard *card);
static void form_work_display_data (RubricaFormWork* form, RubricaCard *card);
static void form_work_clean_form   (RubricaFormWork* form);


static RubricaFormClass* parent_class;


GType
rubrica_form_work_get_type()
{
  static GType form_work_type = 0;
  
  if (!form_work_type)
    {
      static const GTypeInfo form_work_info =
	{
	  sizeof(RubricaFormWorkClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_form_work_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaFormWork),
	  0,
	  (GInstanceInitFunc) rubrica_form_work_init
	};

      form_work_type = g_type_register_static (RUBRICA_FORM_TYPE, 
					       "RubricaFormWork",
					       &form_work_info, 0);
    }
  
  return form_work_type;
}


static void
rubrica_form_work_class_init(RubricaFormWorkClass* klass)
{
  GObjectClass *class;
 
  class  = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) rubrica_form_work_dispose;
  class->finalize = (GObjectFinalizeFunc) rubrica_form_work_finalize;

  parent_class = RUBRICA_FORM_CLASS(klass);
  parent_class->extract_data = (gpointer) form_work_extract_data;
  parent_class->display_data = (gpointer) form_work_display_data;
  parent_class->clean_form   = (gpointer) form_work_clean_form;
}


static void
rubrica_form_work_init(RubricaFormWork* self)
{
  self->private = g_new(RubricaFormWorkPrivate, 1);  

  self->private->role    = NULL;
  self->private->org     = NULL;
  self->private->dep     = NULL;
  self->private->subdep  = NULL;
  self->private->manager = NULL;
  self->private->mphone  = NULL;
  self->private->helper  = NULL;
  self->private->cphone  = NULL;

  self->private->dispose_has_run = FALSE;
}

 

static void 
rubrica_form_work_dispose (RubricaFormWork* self)
{
  g_return_if_fail(IS_RUBRICA_FORM_WORK(self));
  
  if (self->private->dispose_has_run)
    return;
  
  g_object_unref(self->private->role);
  g_object_unref(self->private->org);
  g_object_unref(self->private->dep);
  g_object_unref(self->private->subdep);
  g_object_unref(self->private->manager);
  g_object_unref(self->private->mphone);
  g_object_unref(self->private->helper);
  g_object_unref(self->private->cphone);
  
  self->private->dispose_has_run = TRUE;
}


static void 
rubrica_form_work_finalize (RubricaFormWork* self)
{
  g_return_if_fail(IS_RUBRICA_FORM(self));
  
  g_free(self->private);
  self->private = NULL;
}



static void 
form_work_extract_data (RubricaFormWork* form, RubricaCard *card)
{
  RubricaWork* work;  
  const gchar *role, *org, *dep, *subdep, *manager, *mphone, *helper, *cphone;
  
  g_return_if_fail(IS_RUBRICA_CARD(card));
  
  work = rubrica_work_new();
  
  role    = gtk_entry_get_text(GTK_ENTRY(form->private->role));
  org     = gtk_entry_get_text(GTK_ENTRY(form->private->org));
  dep     = gtk_entry_get_text(GTK_ENTRY(form->private->dep));
  subdep  = gtk_entry_get_text(GTK_ENTRY(form->private->subdep));
  manager = gtk_entry_get_text(GTK_ENTRY(form->private->manager));
  mphone  = gtk_entry_get_text(GTK_ENTRY(form->private->mphone));
  helper  = gtk_entry_get_text(GTK_ENTRY(form->private->helper));
  cphone  = gtk_entry_get_text(GTK_ENTRY(form->private->cphone));
  
  g_object_set(work, "assignment", role, "organization", org,
	       "department", dep, "sub-department", subdep,
	       "manager-name", manager, "manager-phone", mphone,
	       "collaborator", helper, "collaborator-phone", cphone, NULL);

  rubrica_personal_card_set_work(RUBRICA_PERSONAL_CARD(card), work);
}


static void 
form_work_display_data (RubricaFormWork* form, RubricaCard *card)
{
  RubricaWork* work;
  gchar *role, *org, *dep, *subdep, *manager, *mphone, *helper, *cphone;

  work = rubrica_personal_card_get_work (RUBRICA_PERSONAL_CARD(card));

  g_object_get(work, "assignment", &role, "organization", &org,
	       "department", &dep, "sub-department", &subdep,
	       "manager-name", &manager, "manager-phone", &mphone, 
	       "collaborator", &helper, "collaborator-phone", &cphone, NULL);

  if (role)    gtk_entry_set_text(GTK_ENTRY(form->private->role),    role);
  if (org)     gtk_entry_set_text(GTK_ENTRY(form->private->org),     org);
  if (dep)     gtk_entry_set_text(GTK_ENTRY(form->private->dep),     dep);
  if (subdep)  gtk_entry_set_text(GTK_ENTRY(form->private->subdep),  subdep);
  if (manager) gtk_entry_set_text(GTK_ENTRY(form->private->manager), manager);
  if (mphone)  gtk_entry_set_text(GTK_ENTRY(form->private->mphone),  mphone);
  if (helper)  gtk_entry_set_text(GTK_ENTRY(form->private->helper),  helper);
  if (cphone)  gtk_entry_set_text(GTK_ENTRY(form->private->cphone),  cphone);  
}


static void 
form_work_clean_form (RubricaFormWork* form)
{
  gtk_entry_set_text(GTK_ENTRY(form->private->role),    "");
  gtk_entry_set_text(GTK_ENTRY(form->private->org),     "");
  gtk_entry_set_text(GTK_ENTRY(form->private->dep),     "");
  gtk_entry_set_text(GTK_ENTRY(form->private->subdep),  "");
  gtk_entry_set_text(GTK_ENTRY(form->private->manager), "");
  gtk_entry_set_text(GTK_ENTRY(form->private->mphone),  "");
  gtk_entry_set_text(GTK_ENTRY(form->private->helper),  "");
  gtk_entry_set_text(GTK_ENTRY(form->private->cphone),  "");
}


/*   ***************************** Public *****************************
*/


/**
 * rubrica_form_work_new
 *
 * create a #RubricaFormWork
 *
 * Returns: a new #RubricaFormWork
 */
RubricaFormWork*
rubrica_form_work_new(RubricaApp* app, GladeXML *gui)
{
  RubricaFormWork* form;

  form = g_object_new(rubrica_form_work_get_type(), NULL);
  
  form->private->role    = glade_xml_get_widget(gui, "role");
  form->private->org     = glade_xml_get_widget(gui, "org");
  form->private->dep     = glade_xml_get_widget(gui, "dep");
  form->private->subdep  = glade_xml_get_widget(gui, "subdep");
  form->private->manager = glade_xml_get_widget(gui, "manager");
  form->private->mphone  = glade_xml_get_widget(gui, "mphone");
  form->private->helper  = glade_xml_get_widget(gui, "helper");
  form->private->cphone  = glade_xml_get_widget(gui, "cphone");  
  
  return form;
}


/**
 * rubrica_form_work_free
 * @form: a #RubricaFormWork
 * 
 * free the #RubricaFormWork
 */
void
rubrica_form_work_free(RubricaFormWork* form)
{
  g_return_if_fail(IS_RUBRICA_FORM(form));

  g_object_unref(form);   
}
