/*
*  Rubrica
*  file: 
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "groups_cb.h"
#include "utils.h"
#include "types.h"



void on_add_group         (GtkWidget* button, gpointer data);
void on_remove_group      (GtkWidget* button, gpointer data);
void on_entry_activated   (GtkWidget* entry, gpointer data);




void 
on_rubrica_app_groups_event (GtkTreeView* tree, GdkEvent *event, gpointer data)
{
  RubricaApp     *app = (RubricaApp*) data;
  GdkEventButton *ev  = (GdkEventButton *) event;
  RubricaAdb* adb;
  RubricaGroupSet* set;
  GtkTreeModel *model;
  GtkTreeSelection *selection; 
  GtkTreePath *path = NULL;
  GtkTreeIter iter;

  adb       = rubrica_app_get_active_addressbook(app);
  set       = rubrica_app_get_groups(app);
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  selection = gtk_tree_view_get_selection(tree);  
  
  gtk_tree_selection_unselect_all(selection);
  
  if (gtk_tree_view_get_path_at_pos(tree, ev->x, ev->y, &path,
				    NULL, NULL, NULL))
    {
      gtk_tree_selection_select_path(selection, path);
      gtk_tree_model_get_iter(model, &iter, path);
      gtk_tree_path_free(path);

      if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 3))
	{
	  GtkMenu* menu;
	  GtkWidget *remove;
	  GtkWidget *add;	  
	  gchar* user = NULL;
   
	  g_object_set_data(G_OBJECT(set), "tree", tree);
	  gtk_tree_model_get(model, &iter, GROUP_OWNER_COLUMN, &user, -1);
	  
	  menu = (GtkMenu*) gtk_menu_new();
	  
	  remove = gtk_menu_item_new_with_mnemonic ("_remove group");
	  gtk_widget_show (remove);
	  gtk_container_add (GTK_CONTAINER (menu), remove);
	  
	  add = gtk_menu_item_new_with_mnemonic ("add ne_w group");
	  gtk_widget_show (add);
	  gtk_container_add (GTK_CONTAINER (menu), add);
	  
	  if (user && (g_ascii_strcasecmp("user", user) == 0))
	    gtk_widget_set_sensitive(remove, TRUE);
	  else
	    gtk_widget_set_sensitive(remove, FALSE);
	  
	  g_signal_connect (G_OBJECT(add), "activate",
			    G_CALLBACK (on_add_group), set);
	  
	  g_signal_connect (G_OBJECT(remove), "activate",
			    G_CALLBACK (on_remove_group), set);
	  
	  gtk_widget_show_all(GTK_WIDGET(menu));
	  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, ev->button, ev->time);

	}
      else
	if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 1))
	  {
	    gchar* group;

	    gtk_tree_model_get(model, &iter, GROUP_LABEL_COLUMN, &group, -1);
	    // TODO selezionare le schede del gruppo

	    g_print("\ngruppo: %s", group);
	  }
    }
}


void 
on_rubrica_form_groups_event (GtkTreeView* tree, GdkEvent *event, 
			      gpointer data)
{
  RubricaApp     *app = (RubricaApp*) data;
  GdkEventButton *ev  = (GdkEventButton *) event;
  RubricaAdb* adb;
  RubricaGroupSet* usergroups;
  GtkTreeModel *model; 
  GtkTreeSelection *selection;  
  GtkTreePath *path = NULL; 
  GtkTreeIter iter; 

  
  adb        = rubrica_app_get_active_addressbook(app);
  usergroups = rubrica_adb_get_user_groups(adb);
  model      = gtk_tree_view_get_model(GTK_TREE_VIEW(tree)); 
  selection  = gtk_tree_view_get_selection(tree); 
 
  gtk_tree_selection_unselect_all(selection); 
  
  if (gtk_tree_view_get_path_at_pos(tree, ev->x, ev->y, &path,
				    NULL, NULL, NULL))
    {
      gtk_tree_selection_select_path(selection, path);
      gtk_tree_model_get_iter(model, &iter, path);
      gtk_tree_path_free(path);

      if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 3))
	{
	  GtkMenu* menu;
	  GtkWidget *remove;
	  GtkWidget *add;	  
	  gchar* user = NULL;
   
	  g_object_set_data(G_OBJECT(usergroups), "tree", tree);
	  gtk_tree_model_get(model, &iter, GROUP_OWNER_COLUMN, &user, -1);
      
	  menu = (GtkMenu*) gtk_menu_new();
	  
	  remove = gtk_menu_item_new_with_mnemonic ("_remove group");
	  gtk_widget_show (remove);
	  gtk_container_add (GTK_CONTAINER (menu), remove);
	  
	  add = gtk_menu_item_new_with_mnemonic ("add ne_w group");
	  gtk_widget_show (add);
	  gtk_container_add (GTK_CONTAINER (menu), add);
	  
	  if (user && (g_ascii_strcasecmp("user", user) == 0))
	    gtk_widget_set_sensitive(remove, TRUE);
	  else
	    gtk_widget_set_sensitive(remove, FALSE);
	  
	  g_signal_connect (G_OBJECT(add), "activate",
			    G_CALLBACK (on_add_group), usergroups);
	  
	  g_signal_connect (G_OBJECT(remove), "activate",
			    G_CALLBACK (on_remove_group), usergroups);
	  
	  gtk_widget_show_all(GTK_WIDGET(menu));
	  gtk_menu_popup (menu, NULL, NULL, NULL, NULL, ev->button, ev->time);

	}
      else
	if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 1))
	  {
	    gboolean status;      
	    
	    gtk_tree_model_get (model, &iter, 
				GROUP_BELONG_COLUMN, &status, -1); 
	    
	    status ^= 1; 
	    gtk_list_store_set(GTK_LIST_STORE(model), &iter,  
			       GROUP_BELONG_COLUMN, status, -1); 
	  }
    }  
}



void
on_add_group (GtkWidget* button, gpointer data)
{
  GladeXML* gui;  
  RubricaGroupSet* usergroups = (RubricaGroupSet*) data;
  RubricaGroup* group;
  GtkWidget* entry;
  GtkWidget* loadbt;
  GtkWidget* ok;
  GtkWidget* dialog;
  GtkImage* image;
  gchar* str;             /* group name */
  gchar* uri = NULL;      /* pixmap uri */
  gint response;
  
  gui = glade_xml_new (RUBRICA_GUI_DIR"/NewGroup.glade", NULL, NULL);  
  if (!gui)
    g_error("\nCan't load interface");
  
  dialog = glade_xml_get_widget(gui, "newgroup");
  ok     = glade_xml_get_widget(gui, "okbutton1");
  entry  = glade_xml_get_widget(gui, "group_entry");
  loadbt = glade_xml_get_widget(gui, "loadbt");
  image  = (GtkImage*) glade_xml_get_widget(gui, "image");
  g_object_unref(gui);

  g_object_set_data(G_OBJECT(loadbt), "uri", uri);

  g_signal_connect(G_OBJECT(entry), "activate",
		   G_CALLBACK(on_entry_activated), ok);

  g_signal_connect(G_OBJECT(loadbt), "clicked",
		   G_CALLBACK(rubrica_utils_load_pixmap), image);
   
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  switch (response)
    {
    case GTK_RESPONSE_OK:           
      str = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
      uri = g_object_get_data(G_OBJECT(loadbt), "uri");

      group = rubrica_group_new();
      g_object_set(group, "group-name", str, "group-owner", "user",
		   "group-pixmap", uri, "enabled", FALSE, NULL);
      rubrica_group_set_add_group(usergroups, group);
      break;
      
    case GTK_RESPONSE_CANCEL:
      break;
      
    default:
      break;
    }  
  
  gtk_widget_destroy(dialog);  
}


void
on_remove_group (GtkWidget* button, gpointer data)
{
  RubricaGroupSet* usergroups = (RubricaGroupSet*) data;
  RubricaGroup* group;
  GtkTreeView* tree;
  GtkTreeIter iter;
  GtkTreeModel* model;
  GtkTreeSelection *selection;
  gchar* user = NULL;
  gchar* name = NULL;
  
  tree      = g_object_get_data(G_OBJECT(usergroups), "tree"); 
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));

  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, 
			 GROUP_LABEL_COLUMN, &name,
			 GROUP_OWNER_COLUMN, &user,
			 -1);
      
      if (user && (g_ascii_strcasecmp("user", user) == 0))
	{
	  gtk_list_store_remove(GTK_LIST_STORE(model), &iter);	
	  
	  group = rubrica_group_set_find(usergroups, name);
	  if (group)
	    rubrica_group_set_delete_group(usergroups, group);
	}
    }  
}



void 
on_rubrica_add_to_groupstree (RubricaGroupSet *set, gpointer set_data, 
			      gpointer user_data)
{
  GtkTreeView* tree = GTK_TREE_VIEW(user_data);
  GtkTreeIter iter;
  GtkTreeModel* model;   
  RubricaGroup* group = (RubricaGroup*) set_data;
  GdkPixbuf* pixbuf = NULL;
  gchar *uri, *label;
  gboolean enabled;
  
  g_object_get(group, "group-name", &label, "group-pixmap", &uri,
	       "enabled", &enabled, NULL);

  if (uri)
    pixbuf = gdk_pixbuf_new_from_file_at_size (uri, 20, 20, NULL);   

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     GROUP_BELONG_COLUMN,   enabled, 
		     GROUP_PIXMAP_COLUMN,   pixbuf, 
		     GROUP_LABEL_COLUMN,    label,
		     GROUP_OWNER_COLUMN,    "user",
		     GROUP_PIXMAP_PATH,     uri,		     
		     -1);  
}


void 
on_rubrica_remove_from_groupstree (RubricaGroupSet *set, gpointer set_data, 
				   gpointer user_data)
{
  GtkTreeView* tree = GTK_TREE_VIEW(user_data);
  GtkTreeIter iter;
  GtkTreeModel* model;   
  RubricaGroup* group = (RubricaGroup*) set_data;
  GdkPixbuf* pixbuf = NULL;
  gchar *uri, *label;
  gboolean enabled;
  
  g_object_get(group, "group-name", &label, "group-pixmap", &uri, 
	       "enabled", &enabled, NULL);

  if (uri)
    pixbuf = gdk_pixbuf_new_from_file_at_size (uri, 20, 20, NULL);   

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
		     GROUP_BELONG_COLUMN,   enabled, 
		     GROUP_PIXMAP_COLUMN,   pixbuf, 
		     GROUP_LABEL_COLUMN,    label,
		     GROUP_OWNER_COLUMN,    "user",
		     GROUP_PIXMAP_PATH,     uri,		     
		     -1);  
}


void
on_entry_activated(GtkWidget* entry, gpointer data)
{
  g_signal_emit_by_name(G_OBJECT(data), "clicked", NULL);
}
