/*
 *  file: main.c
 *  
 *  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <libral.h>

#include "app.h"
#include "types.h"
#include "utils.h"
#include "callbacks.h"


void 
rubrica_open_addressbooks(RubricaApp* app, gchar* argv[])
{
  RubricaAdb *adb;
  RubricaAbook *book;
  RubricaGroupSet* set;
  
  for (; *argv; argv++)
    {
      adb  = rubrica_adb_new();
      book = rubrica_adb_get_addressbook(adb);
      
      if (rubrica_abook_open_file(RUBRICA_ABOOK(book), *argv))
	{
	  g_print("\nRubrica *** %s opened successfully", *argv);
	  g_print("\nRubrica *** addressbook: %s\nRubrica *** path: %s",
		  rubrica_abook_get_name(RUBRICA_ABOOK(book)),
		  rubrica_abook_get_path(RUBRICA_ABOOK(book)));
	  g_print("\nRubrica *** Cards in this addressbook: %d\n", 
		  rubrica_abook_get_items(RUBRICA_ABOOK(book)));
	  set = rubrica_load_get_user_group();
	  
	  rubrica_adb_set_name(adb, *argv);
	  rubrica_adb_set_user_groups(adb, set);
	  rubrica_app_append_addressbook(app, adb);	  
	}	 
      else
	rubrica_adb_free(adb);
    }
}

int 
main (int argc, char *argv[])
{
  RubricaApp* app = NULL;

  g_thread_init(NULL);
  gtk_init(&argc, &argv);

  g_print("\n\nRubrica ***");
  g_print("\nRubrica *** version: %s", RUBRICA_VERSION); 
  g_print("\nRubrica *** code name: %s, %s",
	  RUBRICA_CODE_NAME, 
	  RUBRICA_INFO_CODE_NAME);
  g_print("\nRubrica ***");
  g_print("\n\nRubrica *** Warning, this is an unstable devel release\n\n"); 
  
  /*  RUBRICA_PIXMAPS_DIR = $(datadir)/pixmaps/rubrica
      ==> /usr/share/gui/rubrica
   */ 
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR);
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/main");  
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/forms");
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/categories");
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/search");
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/view");
  rubrica_utils_add_pixmap_directory(RUBRICA_GUI_DIR"/preferences");
  
  app = rubrica_app_new();

  if (argc > 1)
    rubrica_open_addressbooks(app, ++argv);

  rubrica_app_run(app);
  gtk_main();

  return 0;
}
