/*
*  Rubrica
*  file: 
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <gtk/gtk.h>

#include "utils.h"
#include "types.h"

void rubrica_utils_update_preview(GtkFileChooser *file_chooser, gpointer data);


GList *pixmaps_dir = NULL;



void 
rubrica_utils_add_pixmap_directory(const gchar *directory)
{
  pixmaps_dir = g_list_prepend (pixmaps_dir, g_strdup (directory));
}


gchar*
rubrica_utils_find_pixmap_file (const gchar *filename)
{
  GList *elem;

  if (!filename)
    return NULL;

  /* We step through each of the pixmaps directory to find it. */
  for (elem = pixmaps_dir; elem; elem = elem->next)
    {
      gchar *pathname = g_strdup_printf ("%s%s%s", (gchar*)elem->data, 
					 G_DIR_SEPARATOR_S, filename);

      if (g_file_test (pathname, G_FILE_TEST_EXISTS))
        return pathname;

      g_free (pathname);
    }

  return NULL;
}



GdkPixbuf* 
rubrica_utils_pixbuf_new (gchar *filename)
{
  GdkPixbuf *pix = NULL;
  gchar *imagefile = NULL;
  GError *error = NULL;

  g_return_val_if_fail(filename != NULL, NULL);
  
  if ((g_ascii_strcasecmp(filename, "") == 0)) return NULL;

  imagefile = rubrica_utils_find_pixmap_file(filename);
      
  if (!imagefile)
    {
      g_warning ("Couldn't find pixmap file: %s", filename);

      return NULL;
    }
  
  pix = gdk_pixbuf_new_from_file(imagefile, NULL);
  if (!pix)
    {
      fprintf (stderr, "Failed to load pixbuf file: %s - error: %s\n", 
	       filename, error->message);

      g_error_free (error);
    }  

  g_free(imagefile);
  
  return  pix;
}


void 
rubrica_utils_calendar_ui(GtkWidget* button, gpointer data)
{
  GladeXML* gui = NULL;  
  GtkEntry* entry = (GtkEntry* ) data;
  GtkWidget* dialog;
  GtkWidget* calendar;
  gint response;
  guint year, month, day;
  gchar* str;

  gui = glade_xml_new (RUBRICA_GUI_DIR"/Calendar.glade", NULL, NULL);  
  if (!gui)
    g_error("\nRubrica *** rubrica_utils_calendar_ui: Calendar.glade");

  dialog   = glade_xml_get_widget(gui, "CalendarDialog");
  calendar = glade_xml_get_widget(gui, "calendar");
  g_object_unref(gui);
  
  response = gtk_dialog_run(GTK_DIALOG(dialog));
  switch (response)
    {
    case GTK_RESPONSE_OK:
      gtk_calendar_get_date(GTK_CALENDAR(calendar), &year, &month, &day);

      /*	TODO: localizzare la data
       */
      str = g_strdup_printf("%d-%d-%d", day, month+1, year);
      g_object_set_data(G_OBJECT(entry), "day", (gpointer) day);
      g_object_set_data(G_OBJECT(entry), "month", (gpointer) month);
      g_object_set_data(G_OBJECT(entry), "year", (gpointer) year);
      gtk_entry_set_text(GTK_ENTRY(entry), str);
      g_free(str);
      break;
      
    case GTK_RESPONSE_CANCEL:
      break;
      
    default:
      break;
    }  
  
  gtk_widget_destroy(dialog);
}


void 
rubrica_utils_load_pixmap(GtkWidget* button, gpointer data)
{
  GtkWidget *dialog;
  GtkImage *preview; 
  GtkImage *image = data;
  gchar* uri;

  preview = (GtkImage*) gtk_image_new();

  dialog = gtk_file_chooser_dialog_new ("Open File", NULL,
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					NULL);

  g_object_set_data(G_OBJECT(dialog), "image", image);
  gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (dialog),
				       (GtkWidget*) preview);

  g_signal_connect (GTK_FILE_CHOOSER (dialog), "update-preview",
		    G_CALLBACK (rubrica_utils_update_preview), preview);
  
  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
      uri = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

      g_object_set_data(G_OBJECT(button), "uri", uri);
   }
      
  gtk_widget_destroy (dialog);
}


void
rubrica_utils_update_preview (GtkFileChooser *file_chooser, gpointer data)
{
  GtkWidget *preview;
  GtkImage *image;
  char *filename;
  GdkPixbuf *pixbuf;
  gboolean have_preview;

  preview  = GTK_WIDGET (data);
  image    = g_object_get_data(G_OBJECT(file_chooser), "image");
  filename = gtk_file_chooser_get_preview_filename (file_chooser);

  if (filename)
    {
      pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
      gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
      if (pixbuf)
	gdk_pixbuf_unref (pixbuf);

      pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 48, 48, NULL);
      have_preview = (pixbuf != NULL);
      g_free (filename);

      gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
      if (pixbuf)
	gdk_pixbuf_unref (pixbuf);
      
      gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
    }
}




