/*
*  Rubrica
*  file: view.c
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>
#include <libral.h>
#include <sys/types.h>
#include <unistd.h>

#include "types.h"
#include "gui_types.h"
#include "app.h"
#include "view.h"

void show_header        (GtkWidget* view, RubricaPersonalCard* card);
void show_personal_data (GtkWidget* view, RubricaPersonalCard* card);
void show_groups        (GtkWidget* view, RubricaPersonalCard* card);
void show_addresses     (GtkWidget* view, RubricaPersonalCard* card);
void show_net           (GtkWidget* view, RubricaPersonalCard* card);
void show_telephones    (GtkWidget* view, RubricaPersonalCard* card);
void show_work          (GtkWidget* view, RubricaPersonalCard* card);

void view_show_personal_card (GtkWidget* textview, RubricaPersonalCard* card);
void view_show_company_card  (GtkWidget* textview, RubricaCompanyCard* card);

void wait_please(gchar* type);
void rubrica_view_append_data(GtkWidget* box, gchar* str);


gboolean   hovering_over_link = FALSE;
GdkCursor* hand_cursor        = NULL;
GdkCursor* regular_cursor     = NULL;
GdkCursor* wait_cursor        = NULL;





/* Inserts a piece of text into the buffer, giving it the usual
 * appearance of a hyperlink in a web browser: blue and underlined.
 * Additionally, attaches some data on the tag, to make it recognizable
 * as a link. 
 */
void 
insert_link (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *uri, gchar* type)
{
  GtkTextTag *tag;
  
  tag = gtk_text_buffer_create_tag (buffer, NULL, "foreground", "blue", 
				    "underline", PANGO_UNDERLINE_SINGLE, 
				    NULL);

  g_object_set_data (G_OBJECT (tag), "uri", uri);
  g_object_set_data (G_OBJECT (tag), "type", type);
  
  gtk_text_buffer_insert_with_tags (buffer, iter, uri, -1, tag, NULL);
  gtk_text_buffer_insert_with_tags (buffer, iter, "\n", -1, tag, NULL);
}



/* Looks at all tags covering the position of iter in the text view, 
 * and if one of them is a link, follow it by showing the page identified
 * by the data attached to it.
 */
void
follow_if_link (GtkWidget   *text_view, GtkTextIter *iter, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  GSList *tags = NULL, *tagp = NULL;

  tags = gtk_text_iter_get_tags (iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next) 
    {
      GtkTextTag *tag = tagp->data; 
      gchar* uri = NULL; 
      gchar* type = NULL;
      gchar* command = NULL;
      gchar* arg = NULL;

      uri  = (gchar* ) g_object_get_data (G_OBJECT (tag), "uri"); 
      type = (gchar* ) g_object_get_data (G_OBJECT (tag), "type"); 

      if (uri) 
        {
	  pid_t pid;

	  pid = fork();   
	  if (!pid)        // child
	    {
	      if (g_ascii_strcasecmp(type, "email") == 0)
		{	     
		  command = g_strdup("evolution");
		  arg = g_strdup_printf("mailto:%s", uri);
		}
	      
	      if (g_ascii_strcasecmp(type, "web") == 0)
		
		{
		  command = g_strdup("mozilla"); 
		  arg = g_strdup_printf("%s", uri);
		}
	      
	      
	      if (g_ascii_strcasecmp(type, "gnomemeeting") == 0)
		
		{
		  command = g_strdup("gnomemeeting");
		  arg = g_strdup_printf("-c %s", uri); 
		}
	      
	      if (g_ascii_strcasecmp(type, "icq") == 0)
		
		{
		  command = g_strdup("gaim");
		  arg = g_strdup_printf("-u %s", uri); 

		}
	      
	      //	      wait_please(type);
	      rubrica_app_notify(app, "Please wait. I'm launching client...");
	      if (execlp(command, command, arg, NULL) == -1)
		perror("exec");
	      exit(0);
	    }
	}
    }

  if (tags)  
    g_slist_free (tags);
}


/* Links can be activated by pressing Enter.
 */
gboolean 
key_press_event (GtkWidget *text_view, GdkEventKey *event, gpointer data) 
{
  GtkTextIter iter;
  GtkTextBuffer *buffer;

  switch (event->keyval)
    {
      case GDK_Return: 
      case GDK_KP_Enter:
        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
        gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                                          gtk_text_buffer_get_insert (buffer));
        follow_if_link (text_view, &iter, data);
        break;

      default:
        break;
    }

  return FALSE;
}


/* Links can also be activated by clicking.
 */
gboolean
event_after (GtkWidget *text_view, GdkEvent  *ev, gpointer data)
{
  GtkTextIter start, end, iter;
  GtkTextBuffer *buffer;
  GdkEventButton *event;
  gint x, y;

  if (ev->type != GDK_BUTTON_RELEASE)
    return FALSE;

  event = (GdkEventButton *)ev;

  if (event->button != 1)
    return FALSE;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));

  /* we shouldn't follow a link if the user has selected something */
  gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
  if (gtk_text_iter_get_offset (&start) != gtk_text_iter_get_offset (&end))
    return FALSE;

  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         event->x, event->y, &x, &y);

  gtk_text_view_get_iter_at_location (GTK_TEXT_VIEW (text_view), &iter, x, y);

  follow_if_link (text_view, &iter, data);

  return FALSE;
}



/* Looks at all tags covering the position (x, y) in the text view, 
 * and if one of them is a link, change the cursor to the "hands" cursor
 * typically used by web browsers.
 */
void
set_cursor_if_appropriate (GtkTextView *text_view, gint x, gint y)
{
  GSList *tags = NULL, *tagp = NULL;
  GtkTextBuffer *buffer;
  GtkTextIter iter;
  gboolean hovering = FALSE;

  buffer = gtk_text_view_get_buffer (text_view);

  gtk_text_view_get_iter_at_location (text_view, &iter, x, y);
  
  tags = gtk_text_iter_get_tags (&iter);
  for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
    {
      GtkTextTag *tag = tagp->data;
      gchar* str = NULL;
	
      str = g_object_get_data (G_OBJECT (tag), "uri");
      if (str) 
        {
          hovering = TRUE;
          break;
        }
    }

  if (hovering != hovering_over_link)
    {
      GdkWindow* text_window;

      hovering_over_link = hovering;
      
      text_window = gtk_text_view_get_window(text_view, GTK_TEXT_WINDOW_TEXT);
      if (hovering_over_link)
        gdk_window_set_cursor(text_window, hand_cursor);
      else
        gdk_window_set_cursor(text_window, regular_cursor);
    }

  if (tags) 
    g_slist_free (tags);
}


/* Update the cursor image if the pointer moved. 
 */
gboolean
motion_notify_event (GtkWidget *text_view, GdkEventMotion *event,
		     gpointer data)
{
  gint x, y;

  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         event->x, event->y, &x, &y);

  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), x, y);

  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
  return FALSE;
}


/* Also update the cursor image if the window becomes visible
 * (e.g. when a window covering it got iconified).
 */
gboolean
visibility_notify_event (GtkWidget *text_view, GdkEventVisibility *event, 
			 gpointer data)
{
  gint wx, wy, bx, by;
  
  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
  
  gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                         GTK_TEXT_WINDOW_WIDGET,
                                         wx, wy, &bx, &by);

  set_cursor_if_appropriate (GTK_TEXT_VIEW (text_view), bx, by);

  return FALSE;
}


/* ----------------------- FIXME -------------------------------- */

void wait_please(gchar* type)
{
  GtkWidget *win, *w, *vbox;
  guint sid;
  gchar* message = NULL;
  
  /* create the modal window which warns the user to wait */
  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(win), TRUE);
  gtk_window_set_title(GTK_WINDOW(win), "Please wait...");
  gtk_container_set_border_width(GTK_CONTAINER(win), 12);
  g_signal_connect(win, "delete_event", G_CALLBACK(gtk_true), NULL);

  vbox = gtk_vbox_new(FALSE, 12);
  gtk_widget_show(vbox);
  
  /* create label 
   */
  if (g_ascii_strcasecmp(type, "email") == 0)
    message = g_strdup("Launching email client.\nPlease wait...");
  
  if (g_ascii_strcasecmp(type, "web") == 0)
    message = g_strdup("Launching browser web client.\nPlease wait...");

  if (g_ascii_strcasecmp(type, "gnomemeeting") == 0)
    message = g_strdup("Launching video conferencing client.\nPlease wait...");

  if (g_ascii_strcasecmp(type, "icq") == 0)
    message = g_strdup("Launching irc client.\nPlease wait...");

  w = gtk_label_new(message);
  gtk_widget_show(w);
  gtk_container_add(GTK_CONTAINER(vbox), w);
  
  /* create progress bar */
  w = gtk_progress_bar_new();
  gtk_widget_show(w);
  gtk_container_add(GTK_CONTAINER(vbox), w);
  
  /* add vbox to dialog */
  gtk_container_add(GTK_CONTAINER(win), vbox);
  gtk_widget_show(win);
  
  /* refresh the progress dialog
  sid = g_timeout_add(300, update_progress, w);
  g_object_set_data(G_OBJECT(win), "source_id", GINT_TO_POINTER(sid));
 
  g_thread_create(async_func, win, FALSE, NULL); 
 */ 
  g_free(message);
}

/* ********************************************************************* */



void show_header(GtkWidget* view, RubricaPersonalCard* card)
{
  gchar* card_name, *str;
  GtkTextBuffer* buffer;
  GtkTextIter iter;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  g_object_get(RUBRICA_CARD(card), "card-name", &card_name, NULL);
  gtk_text_buffer_get_iter_at_offset(buffer, &iter, 0);
  
  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, _("Card: "), 
					   -1, "header", "bold", NULL);  
  str = g_strdup_printf("%s\n", card_name);  
  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, str, -1, 
					   "data", NULL); 
  g_free(str);
  
  gtk_text_buffer_insert(buffer, &iter, "\n\n", -1);
}


void 
show_personal_data(GtkWidget* view, RubricaPersonalCard* card)
{
  RubricaContact* contact;
  GtkWidget* dataview;
  GtkTextBuffer* databuffer;
  GtkTextBuffer* buffer;
  GtkTextIter start, end, iter;
  GtkTextMark *cursor;
  GtkTextChildAnchor *anchor;  
  GtkWidget* image;
  GdkPixbuf* pixbuf = NULL;  
  GtkWidget* hbox;
  GError *error = NULL;
  gchar *day, *month, *year;
  gchar *first, *last, *middle, *nick, *title, *prefix, *prof, *photo;
      
  /* get cursor position and insert an anchor for widget insertion 
   */
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  cursor = gtk_text_buffer_get_insert (buffer);
  gtk_text_buffer_get_iter_at_mark (buffer, &iter, cursor);
  anchor = gtk_text_buffer_create_child_anchor (buffer, &iter);
  gtk_text_buffer_insert (buffer, &iter, "\n\n", -1);  
  
  
  /* Data view
   */
  hbox = gtk_hbox_new(FALSE, 6);
  gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(view), hbox, anchor);
  gtk_widget_show(hbox);
  
  contact = rubrica_personal_card_get_contact(RUBRICA_PERSONAL_CARD(card));

  g_object_get(contact, "first-name", &first, "last-name", &last, 
	       "middle-name", &middle, "nick-name", &nick, 
	       "title", &title, "prefix", &prefix, 
	       "profession", &prof, "photo", &photo, NULL);

  if (!photo || !g_file_test(photo, G_FILE_TEST_EXISTS))
    photo = g_strdup(RUBRICA_GUI_DIR"/view/photo.png");

  pixbuf = gdk_pixbuf_new_from_file_at_size (photo, 124, 124, &error);
  if (pixbuf)
    {
      image = gtk_image_new_from_pixbuf(pixbuf);
      gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 6);
      
      gtk_widget_show(image);

      g_free(photo);
    }
    
  dataview = gtk_text_view_new();
  gtk_text_view_set_editable(GTK_TEXT_VIEW(dataview), FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(dataview), FALSE);
  databuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dataview)); 
  gtk_text_buffer_create_tag (databuffer, "bold", "weight", 
			      PANGO_WEIGHT_BOLD, NULL);
  gtk_box_pack_start(GTK_BOX(hbox), dataview, TRUE, TRUE, 0);
  gtk_widget_show(dataview);
  
  gtk_text_buffer_get_bounds(databuffer, &start, &end);
  gtk_text_buffer_delete(databuffer, &start, &end);
  gtk_text_buffer_get_iter_at_offset(databuffer, &start, 0);  
  
  if (first || middle || last)
    {
      gchar *str;
      
      str = g_strdup_printf("%s %s %s", 
			    last   ? last   : "", 
			    first  ? first  : "", 
			    middle ? middle : "");
      
      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, _("Name: "),
					       -1, "bold", NULL);        
      gtk_text_buffer_insert (databuffer, &start, str, -1);
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       
      
      g_free(last);
      g_free(first);
      g_free(middle);
      g_free(str);
    }

  if (nick && (g_ascii_strcasecmp(nick, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, 
					       _("Nickname: "), -1, 
					       "bold", NULL);
      gtk_text_buffer_insert (databuffer, &start, nick, -1); 
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       
      g_free(nick);
    }

  if (title && (g_ascii_strcasecmp(title, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, 
					       _("Title: "), -1, "bold", NULL);
      gtk_text_buffer_insert (databuffer, &start, title, -1); 
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       
      g_free(title);
    }
  
  if (prefix && (g_ascii_strcasecmp(prefix, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, 
					       _("Prefix: "), -1,
					       "bold", NULL);
      gtk_text_buffer_insert (databuffer, &start, prefix, -1);
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       
      g_free(prefix);
    }
  
  if (prof && (g_ascii_strcasecmp(prof, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, 
					       _("Profession: "), -1, 
					       "bold", NULL);  
      gtk_text_buffer_insert (databuffer, &start, prof, -1);
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       
      g_free(prof);
    }
  
  day   = rubrica_contact_get_birth_day(contact);
  month = rubrica_contact_get_birth_month(contact);
  year  = rubrica_contact_get_birth_year(contact);  
  if ((g_ascii_strcasecmp(day, "0") == 0)     ||
      ((g_ascii_strcasecmp(month, "0") == 0)) ||
      ((g_ascii_strcasecmp(year, "0") == 0)))
    {
      g_free(day);
      g_free(month);
      g_free(year);
    }
  else
    {
      gchar *str;

      gtk_text_buffer_insert_with_tags_by_name(databuffer, &start, 
					       _("Birthday: "), -1, 
					       "bold", NULL);
      
      str = g_strdup_printf("%s-%s-%s", day, month, year);     
      gtk_text_buffer_insert (databuffer, &start, str, -1);
      gtk_text_buffer_insert (databuffer, &start, "\n", -1);       

      g_free(day);
      g_free(month);
      g_free(year);
      g_free(str);
    }
}


void 
show_groups(GtkWidget* view, RubricaPersonalCard* card)
{
  GtkTextIter iter;
  GtkTextBuffer* buffer;
  gpointer data;
  
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  
  gtk_text_buffer_get_end_iter(buffer, &iter);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
					    _("Card's groups\n"), -1,
					    "bold", NULL);
  data = rubrica_card_get_group(RUBRICA_CARD(card));
  for (; data; data = rubrica_card_get_next_group(RUBRICA_CARD(card)))
    {
      gchar *str, *name, *owner;
      
      g_object_get(RUBRICA_GROUP(data), "group-name", &name, 
		   "group-owner", &owner, NULL);

      str = g_strdup_printf(_("\t%s (owner: %s)\n"), name, owner);
      gtk_text_buffer_insert (buffer, &iter, str, -1);

      g_free(str);
      g_free(name);
      g_free(owner);
    }  
}


void 
show_addresses(GtkWidget* view, RubricaPersonalCard* card)
{
  RubricaAddress* address;
  GtkTextBuffer* buffer;
  GtkTextIter iter;
  GdkPixbuf* pixbuf = NULL;  
  gchar *str;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));

  gtk_text_buffer_get_end_iter(buffer, &iter);
  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR"/forms/"
					    "mailbox.png", 24, 24, NULL);
  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
  
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("Addresses\n"),
					    -1, "bold", NULL);

  address = rubrica_card_get_address(RUBRICA_CARD(card));
  for (; address; address = rubrica_card_get_next_address(RUBRICA_CARD(card)))
    {
      RubricaAddressType type;
      gchar *adtype, *street, *number, *zip;
      gchar *city, *province, *state, *country;
      
      g_object_get(address, "address-type", &type, "street", &street, 
		   "street-number", &number, "zip", &zip, "city", &city,
		   "province", &province, "state", &state, 
		   "country", &country, NULL);

      gtk_text_buffer_insert (buffer, &iter, "\t", -1);
      pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR"/forms/"
						"mail.png", 24, 24, NULL);
      gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);

      adtype = rubrica_address_decode_type(type);
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, adtype, -1, 
						"bold", NULL);
      
      str = g_strdup_printf("\n\t%s, %s\n\t%s %s, (%s)\n\t%s, %s",
			    street, number, zip, city,
			    province, state, country);
      gtk_text_buffer_insert (buffer, &iter, str, -1);
      g_free(str);     
      gtk_text_buffer_insert (buffer, &iter, "\n", -1);
    }
}


void 
show_work(GtkWidget* view, RubricaPersonalCard* card)
{
  RubricaWork* work;
  GtkTextBuffer* buffer;
  GtkTextIter iter;
  GdkPixbuf* pixbuf = NULL;  
  gchar *assignment, *organization, *department, *subdep, *manager;
  gchar *collaborator, *mphone, *cphone;


  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_get_end_iter(buffer, &iter);

  gtk_text_buffer_insert (buffer, &iter, "\n", -1);
  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					 "/forms/role.png", 24, 24, NULL);
  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("Work"), -1,
					    "bold", NULL);
  
  work = rubrica_personal_card_get_work(RUBRICA_PERSONAL_CARD(card));
  g_object_get(work, "assignment", &assignment, "organization", &organization, 
	       "department", &department, "sub-department", &subdep, 
	       "manager-name", &manager, "collaborator", &collaborator,
	       "manager-phone", &mphone, "collaborator-phone", &cphone, NULL);
	       

  if (assignment && (g_ascii_strcasecmp(assignment, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tAssignment: "), -1, 
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, assignment, -1);
      
      g_free(assignment);
    }
  
  if (organization && (g_ascii_strcasecmp(organization, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tOrganization: "), -1, 
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, organization, -1);
      
      g_free(organization);
    }

  if (department && (g_ascii_strcasecmp(department, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tDepartment: "), -1,
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, department, -1);
      
      g_free(department);
    }

  if (subdep && (g_ascii_strcasecmp(subdep, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tSubdepartment: "), -1,
						"bold", NULL);
       gtk_text_buffer_insert(buffer, &iter, subdep, -1);
       
       g_free(subdep);
    }

  if (manager && (g_ascii_strcasecmp(manager, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tManager: "), -1, 
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, manager, -1);

      g_free(manager);
    }

  if (mphone && (g_ascii_strcasecmp(mphone, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter,
						_("\n\tManager's telephone: "),
						-1, "bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, mphone, -1);
      
      g_free(mphone);
    }

  if (collaborator && (g_ascii_strcasecmp(collaborator, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tCollaborator: "), -1,
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, collaborator, -1);

      g_free(collaborator);
    }

  if (cphone && (g_ascii_strcasecmp(cphone, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tCollaborator's"
						  " telephone: "),
						-1, "bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, cphone, -1);

      g_free(cphone);
    }
}



void 
show_net (GtkWidget* view, RubricaPersonalCard* card)
{
  GtkTextBuffer* buffer;
  GtkTextIter iter;
  GdkPixbuf* pixbuf = NULL;  
  RubricaNetAddress* net;
  RubricaNetAddressType type;
  gchar* url;
  gchar *str;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_get_end_iter(buffer, &iter);

  gtk_text_buffer_insert (buffer, &iter, "\n\n", -1);
  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					    "/forms/internet.png", 
					    24, 24, NULL);
  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("Net\n"), -1,
					    "bold", NULL);

 
  for (net = rubrica_card_get_net_address(RUBRICA_CARD(card)); net; 
       net = rubrica_card_get_next_net_address(RUBRICA_CARD(card)))
    {

      
      g_object_get(RUBRICA_NET_ADDRESS(net), "url", &url, 
		   "url-type", &type, NULL);

      str = g_strdup_printf(_("\t%s: "), 
			    rubrica_net_address_decode_type(type));

      switch(type)
	{
	case RUBRICA_NET_ADDRESS_WEB:
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/gnome-globe.png",
						    20, 20, NULL);
	  break;

	case RUBRICA_NET_ADDRESS_EMAIL:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/email.png",
						    20, 20, NULL);
	  break;

	case RUBRICA_NET_ADDRESS_GNOMEMEETING:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/gnomemeeting.png",
						    20, 20, NULL);
	  break;
	  
	case RUBRICA_NET_ADDRESS_IRC_AIM:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/aim.png",
						    20, 20, NULL);
	  break;
	  
	case RUBRICA_NET_ADDRESS_IRC_ICQ:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/icq.png",
						    20, 20, NULL);
	  break;
	  
	case RUBRICA_NET_ADDRESS_IRC_JABBER:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/jabber.png",
						    20, 20, NULL);
	  break;
	  
	case RUBRICA_NET_ADDRESS_IRC_YAHOO:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/yahoo.png",
						    20, 20, NULL);
	  break;

	case RUBRICA_NET_ADDRESS_IRC_MSN:  
	  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
						    "/view/msn.png",
						    20, 20, NULL);
	  break;


	default:
	  break;
	}
      
      gtk_text_buffer_insert (buffer, &iter, "\t", -1);      
      gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);		    
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, str, -1,
						"bold", NULL);      
      g_free(str);
      
      str = rubrica_net_address_decode_type(type);
      insert_link(buffer, &iter, url, str);   
    }
}


void 
show_telephones(GtkWidget* view, RubricaPersonalCard* card)
{
  RubricaTelephone* tel;
  GtkTextBuffer* buffer;
  GtkTextIter iter;
  GdkPixbuf* pixbuf = NULL;  
  gchar *str;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));

  gtk_text_buffer_get_end_iter(buffer, &iter);
  gtk_text_buffer_insert (buffer, &iter, "\n\n", -1);
  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					    "/forms/telephone.png",
					    24, 24, NULL);
  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("Telephone"), -1,
					    "bold", NULL);

  tel = rubrica_card_get_telephone(RUBRICA_CARD(card));
  for (; tel; tel = rubrica_card_get_next_telephone(RUBRICA_CARD(card)))
    {
      RubricaTelephoneType ttype;
      gchar* num;
      gchar* type;

      g_object_get(RUBRICA_TELEPHONE(tel), 
		   "telephone-number", &num,
		   "telephone-type", &ttype, NULL);
      type = rubrica_telephone_decode_type(ttype);

      str = g_strdup_printf("\n\t%s: ", type);
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, str, -1,
						"bold", NULL);
      gtk_text_buffer_insert (buffer, &iter, num, -1);
      g_free(str);    	
    }
}


void
show_notes(GtkWidget* view, RubricaPersonalCard* card)
{
  RubricaNotes* notes;
  GtkTextBuffer* buffer;
  GtkTextIter iter;
  GdkPixbuf* pixbuf = NULL;  
  gchar *partner, *children, *other_notes, *pubkey;
  gboolean has_partner;

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));

  gtk_text_buffer_get_end_iter(buffer, &iter);
  gtk_text_buffer_insert (buffer, &iter, "\n\n", -1);
  pixbuf = gdk_pixbuf_new_from_file_at_size(RUBRICA_GUI_DIR
					    "/forms/notes.png", 24, 24, NULL);
  gtk_text_buffer_insert_pixbuf(buffer, &iter, pixbuf);
  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, _("Notes"), -1,
			  "bold", NULL);
  
  notes = rubrica_personal_card_get_notes(RUBRICA_PERSONAL_CARD(card));

  g_object_get(notes, "has-partner", &has_partner, "partner-name", &partner,
	       "children", &children, "other-notes", &other_notes, 
	       "pubkey", &pubkey, NULL);

  if (has_partner)
    {   
      gtk_text_buffer_insert (buffer, &iter, 
			      _("\n\tContact has a partner"), -1);
      
      if (partner && (g_ascii_strcasecmp(partner, "") != 0))
	{
	  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter,
						    _("\n\tpartner name: "),
						    -1, "bold", NULL);
	  gtk_text_buffer_insert(buffer, &iter, partner, -1);
	}
      
      if (rubrica_notes_know_birthday(notes))
	{	  
	  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
						   _("\n\tpartner birthday: "),
						   -1, "bold", NULL);
	  gtk_text_buffer_insert(buffer, &iter, 
				 rubrica_notes_get_birthdate(notes), -1);
	}
      
      if (rubrica_notes_know_anniversary(notes))
	{
	  gtk_text_buffer_insert_with_tags_by_name (buffer, &iter,
						    _("\n\tanniversary: "), -1,
						    "bold", NULL);
	  gtk_text_buffer_insert(buffer, &iter, 
				 rubrica_notes_get_anniversarydate(notes), -1);
	}
    }

  if (pubkey  && (g_ascii_strcasecmp(pubkey, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tPublic key: "), -1,
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, pubkey, -1);

      g_free(pubkey);
    }

  if (other_notes  && (g_ascii_strcasecmp(other_notes, "") != 0))
    {
      gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, 
						_("\n\tOther notes: "), -1,
						"bold", NULL);
      gtk_text_buffer_insert(buffer, &iter, other_notes, -1);

      g_free(other_notes);
    } 
}


void
rubrica_view_show_personal_card(GtkWidget* view, RubricaPersonalCard* card)
{
  show_header(view, card);
  show_personal_data(view, card);
  show_groups(view, card);
  show_addresses (view, card);
  show_work(view, card);
  show_net(view, card);
  show_telephones(view, card);
  show_notes(view, card);
}



void
rubrica_view_show_company_card(GtkWidget* textview, RubricaCompanyCard* card)
{
}


void 
rubrica_view_show_data (RubricaApp* app, gpointer data)
{
  RubricaCard* card = (RubricaCard*) data;
  GtkWidget* view;
  GtkTextBuffer* buffer;
  GtkTextIter start;
  GtkTextIter end;
  RubricaRank rank;
  gboolean deletable;
  gchar* type;

  g_return_if_fail(IS_RUBRICA_CARD(card));

  view = rubrica_app_get_textview(app);
  g_object_get(card, "card-type", &type, "deletable", &deletable,
	       "rank", &rank, NULL);
  
  /* clean view */
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
  gtk_text_buffer_get_bounds(buffer, &start, &end);
  gtk_text_buffer_delete(buffer, &start, &end);
  gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);  
  
  rubrica_app_enable_rank_pixmap(app, rank);
  if (deletable)
    rubrica_app_enable_lock_pixmap(app, PIXMAP_UNLOCK);
  else
    rubrica_app_enable_lock_pixmap(app, PIXMAP_LOCK);
   
  if (g_ascii_strcasecmp(type, "personal") == 0)
    rubrica_view_show_personal_card(view, RUBRICA_PERSONAL_CARD(card));
  else
    rubrica_view_show_company_card(view, RUBRICA_COMPANY_CARD(card));
}

 
void rubrica_view_init_view(RubricaApp* app, GtkWidget* view)
{
  PangoFontDescription* pfd;   /* pango font description */
  GtkTextBuffer* buffer;
  GtkTextIter start;
  GtkTextIter end;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (view));

  /* default fonts
   */
  gtk_text_buffer_create_tag (buffer, "bold", "weight", 
			      PANGO_WEIGHT_BOLD, NULL);
  pfd  = pango_font_description_from_string ("Tines 14");    
  gtk_text_buffer_create_tag (buffer, "header", "font-desc", pfd, NULL);
  pango_font_description_free (pfd);   
  
  pfd  = pango_font_description_from_string ("Tines 10");    
  gtk_text_buffer_create_tag (buffer, "data", "font-desc", pfd, NULL);
  pango_font_description_free (pfd);   

  /* clean textview 
   */
  gtk_text_buffer_get_bounds(buffer, &start, &end);
  gtk_text_buffer_delete(buffer, &start, &end);
  gtk_text_buffer_get_iter_at_offset(buffer, &start, 0);  

  /* cursor's pixmaps
   */
  hand_cursor    = gdk_cursor_new (GDK_HAND2); 
  regular_cursor = gdk_cursor_new (GDK_XTERM);
  wait_cursor    = gdk_cursor_new (GDK_WATCH);
  
  /* handle events
   */
  g_signal_connect (G_OBJECT (view), "key-press-event", 
		    G_CALLBACK (key_press_event), app);

  g_signal_connect (G_OBJECT (view), "event-after", 
		    G_CALLBACK (event_after), app);

  g_signal_connect (G_OBJECT (view), "motion-notify-event", 
		    G_CALLBACK (motion_notify_event), app);

  g_signal_connect (G_OBJECT (view), "visibility-notify-event", 
		    G_CALLBACK (visibility_notify_event), app);
}
